/*****************************************************************************
 * File:  sys_sets.h
 *
 *
 * Notice:
 *   (C) Copyright 1999-2003 ROX Software, Inc.
 *   All rights reserved.
 *
 * Model Compiler:  MC3020 V3.3.0
 * Serial Number:   302033003113005
 *
 * Warnings:
 *   !!! THIS IS AN AUTO-GENERATED FILE. PLEASE DO NOT EDIT. !!!
 ****************************************************************************/

#ifndef SYS_SETS_H
#define SYS_SETS_H

#ifdef	__cplusplus
extern "C" {
#endif


#include "sys_types.h"

/*****************************************************************************
 * This is the most simple and basic container class and
 * represents elements in the set (nodes in the list).
 ****************************************************************************/
typedef struct Escher_SetElement_s Escher_SetElement_s;
struct Escher_SetElement_s
{
  Escher_SetElement_s * next;
  void * object;
};

/*****************************************************************************
 * This is the set class and may contain attributes that apply
 * to the set as a whole.
 ****************************************************************************/
typedef struct Escher_ObjectSet_s Escher_ObjectSet_s;
struct Escher_ObjectSet_s
{
  Escher_SetElement_s * head;
};

/*****************************************************************************
 * Notice the symmetry between the object containoid, the
 * head of set structure and the set node containoid.
 ****************************************************************************/
typedef struct Escher_Object_s Escher_Object_s;
struct Escher_Object_s
{
  Escher_SetElement_s * next;
};

/*****************************************************************************
 * Initialize the node1 instances by linking them into a collection.
 * These containoids will be collected into a null-terminated,
 * singly linked list (slist).
 * This needs to be called during architecture initialization.
 ****************************************************************************/
extern void Escher_SetFactoryInit( void );

/*****************************************************************************
 * This will copy all of the elements from one set into another
 * set.  If the target set is not empty, it will be cleared
 * before the copy operation occurs freeing any nodes in that set.
 * The new set will use containoids from the free list.
 ****************************************************************************/
extern void Escher_CopySet( Escher_ObjectSet_s *,
                Escher_ObjectSet_s * const );

/*****************************************************************************
 * Release all nodes in the given set back to the free pool.
 ****************************************************************************/
extern void Escher_ClearSet( Escher_ObjectSet_s * );

/*****************************************************************************
 * Insert a single element into the set in no particular order.
 * The element is a data item.  A container node will be allocated
 * to link in the element.
 ****************************************************************************/
extern void Escher_SetInsertElement( Escher_ObjectSet_s *,
                         void * const );

/*****************************************************************************
 * Insert a block of objects into the given set in sequence.  Link the
 * data into the supplied containers.
 * Return a pointer to the linked set.
 ****************************************************************************/
extern Escher_SetElement_s *
Escher_SetInsertBlock( Escher_SetElement_s *,
                       const u1_t *,
                       const u2_t,
                       s2_t );

/*****************************************************************************
 * Remove an instance from an instance collection.
 ****************************************************************************/
#define Escher_SetRemoveInstance( pextent, instance, slot, container, pool ) \
  slot = Escher_SetRemoveNode( pextent, instance )

/*****************************************************************************
 * Insert an instance onto an extent.
 ****************************************************************************/
#define Escher_SetInsertInstance( pextent, node ) \
  node->next = (pextent)->head; \
  (pextent)->head = node


/*****************************************************************************
 * Remove a data item from the given set.  This requires searching
 * the set for the item, unlinking the item (if found) and returning
 * the pointer to the removed node.  SetRemoveNode is used
 * when some knowledge of the linking mechanism is required (as
 * in extent management).  SetRemoveElement is used whenever
 * possible.
 ****************************************************************************/
extern Escher_SetElement_s *
Escher_SetRemoveNode( Escher_ObjectSet_s *,
                      const void * const );

/*****************************************************************************
 * This interface is used more often to remove an element.  This is
 * used when maximum anonymity is required.  Escher_SetRemoveNode is
 * used when some knowledge of the linking mechanism is required (as
 * in extent management).
 ****************************************************************************/
extern void Escher_SetRemoveElement( Escher_ObjectSet_s *,
                         const void * const );

/*****************************************************************************
 * Return true when the set contains the given data element.
 * Return false if not found.
 ****************************************************************************/
extern bool Escher_SetContains( const Escher_ObjectSet_s * const,
                    const void * const );

/*****************************************************************************
 * Count the elements in the set.  Return that count.
 * This routine counts nodes.
 ****************************************************************************/
extern u2_t Escher_SetCardinality( const Escher_ObjectSet_s * const );

/*****************************************************************************
 * Return true when the left and right set are equivalent.
 * Note:  This currently is not implemented.
 ****************************************************************************/
extern bool Escher_SetEquality( Escher_ObjectSet_s * const,
                    Escher_ObjectSet_s * const );

/*****************************************************************************
 * Initialize a set variable.
 ****************************************************************************/
/* Signature:  void Escher_InitSet( Escher_ObjectSet_s * set ) */
#define Escher_InitSet( S ) (S)->head = 0

/*****************************************************************************
 * Get any element (1st) from a collection.
 ****************************************************************************/
/* Signature:  void * Escher_SetGetAny( Escher_ObjectSet_s * const set ) */
#define Escher_SetGetAny( S ) ( ((S)->head != 0) ? (S)->head->object : 0 )

/*****************************************************************************
 * Return true if set is empty.
 ****************************************************************************/
/* Signature:  bool Escher_SetIsEmpty( const Escher_ObjectSet_s * const set ) */
#define Escher_SetIsEmpty( S ) ( (S)->head == 0 )

/*****************************************************************************
 * Collection Iteration
 * The iterator uses a cursor external to the set (or extent).
 * This cursor maintains a flavor of current element.
 ****************************************************************************/
typedef struct Escher_Iterator_s Escher_Iterator_s;
struct Escher_Iterator_s
{
  Escher_SetElement_s * cursor;
};

/*****************************************************************************
 * Use this method to reset the cursor.
 ****************************************************************************/
/* Signature:  void Escher_IteratorReset( Escher_Iterator_s * const iterator,
 *                                        Escher_ObjectSet_s * const set )
 */
#define Escher_IteratorReset( I, S ) ( ((I)->cursor = (S)->head) )

/*****************************************************************************
 * Interate to the next element and return it.
 ****************************************************************************/
extern void * Escher_IteratorNext( Escher_Iterator_s * const );

/*****************************************************************************
 * Measure the length of the given string.
 ****************************************************************************/
/* We could easily replace this function declaration with a macro
   that invoked the compiler (C library) supplied strlen.  */
extern u2_t Escher_strlen( const c_t * );

/*****************************************************************************
 * Set memory bytes to value at destination.
 ****************************************************************************/
/* We could easily replace this function declaration with a macro
   that invoked the compiler (C library) supplied memset.  */
extern void Escher_memset( void * const, const u1_t, u2_t );

/*****************************************************************************
 * Move memory bytes from source to destination.
 ****************************************************************************/
/* We could easily replace this function declaration with a macro
   that invoked the compiler (C library) supplied memmove.  */
extern void Escher_memmove( void * const, const void * const, u2_t );

/*****************************************************************************
 * Copy characters and be paranoid about null delimiter.
 ****************************************************************************/
extern void Escher_strcpy( c_t *, const c_t * );

/*****************************************************************************
 * Add two strings allowing for commutativity.  Using shared area,
 * so limited to single threaded and simple string expressions.
 ****************************************************************************/
extern c_t * Escher_stradd( const c_t *, const c_t * );

/*****************************************************************************
 * Compare two strings.
 * Return negative number if s1 < s2.
 * Return zero if s1 == s2.
 * Return postive number if s1 > s2.
 ****************************************************************************/
/* We could easily replace this function declaration with a macro
   that invoked the compiler (C library) supplied strcmp.  */
extern c_t Escher_strcmp( const c_t *, const c_t * );


#ifdef	__cplusplus
}
#endif

#endif  /* SYS_MECHS_H */

