/*---------------------------------------------------------------------------
 * File:  tm_layer.h
 *
 * Description:
 * The routines in this file provide a layer of indirection between
 * the fragment code and the UML Debugger protocol layer.
 *
 * This interface layer originally specified by Project Technology, Inc.
 *--------------------------------------------------------------------------*/
#ifndef TM_LAYER_H
#define TM_LAYER_H

#ifdef	__cplusplus
extern "C" {
#endif
#ifdef ROX_TM_ENABLED
void ROX_TMProtocol_Add_Domain(unsigned long dom_id, const char *xml_url);
void ROX_TMProtocol_Log_Break_execution(int stmt_id);
void ROX_TMProtocol_Log_Continue_execution(int stmt_id);
void ROX_TMProtocol_Log_Control_execution(int stmt_id, const char *command);
void ROX_TMProtocol_Log_Creation_Transition(unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, int num_params, unsigned long attr_id[], const char *value[]);
bool ROX_TMProtocol_Log_Elif_execution(int stmt_id, const char *condition, bool eval);
void ROX_TMProtocol_Log_Else_execution(int stmt_id);
void ROX_TMProtocol_Log_Endfor_execution(int stmt_id);
void ROX_TMProtocol_Log_Endif_execution(int stmt_id);
void ROX_TMProtocol_Log_Endwhile_execution(int stmt_id);
void ROX_TMProtocol_Log_Event_Cant_Happen(unsigned long dom_id, unsigned long sm_id, unsigned long smstt_id, unsigned long obj_inst_id, unsigned long smevt_id, unsigned long evt_inst_id);
void ROX_TMProtocol_Log_Event_Ignored(unsigned long dom_id, unsigned long sm_id, unsigned long smstt_id, unsigned long obj_inst_id, unsigned long smevt_id, unsigned long evt_inst_id);
bool ROX_TMProtocol_Log_For_execution(int stmt_id, const char *loop_variable_name, const char *set_variable_name, int is_implicit, bool eval);
bool ROX_TMProtocol_Log_If_execution(int stmt_id, const char *condition, bool eval);
void ROX_TMProtocol_Log_Occurrence(const char *severity, const char *message);
void ROX_TMProtocol_Log_Return_execution(int stmt_id, const char *return_value, const char *oal_value);
void ROX_TMProtocol_Log_Select_from_execution(int stmt_id, const char *cardinality, const char *key_lett, int is_implicit, const char *local_variable);
void ROX_TMProtocol_Log_Select_from_where_execution(int stmt_id, const char *cardinality, const char *key_lett, const char *where_clause, int is_implicit, const char *local_variable);
void ROX_TMProtocol_Log_Select_related_by_execution(int stmt_id, const char *cardinality, unsigned long obj_inst_id, const char *chain, int is_implicit, const char *local_variable);
void ROX_TMProtocol_Log_Select_related_by_where_execution(int stmt_id, const char *cardinality, unsigned long obj_inst_id, const char *chain, const char *where_clause, int is_implicit, const char *local_variable);
bool ROX_TMProtocol_Log_While_execution(int stmt_id, const char *condition, bool eval);
void ROX_TMProtocol_Notify_Associative_Relationship_Creation(int stmt_id, unsigned long dom_id, unsigned long rel_id, unsigned long from_obj_inst_id, const char *from_obj_inst_variable, unsigned long to_obj_inst_id, const char *to_obj_inst_variable, unsigned long assoc_obj_inst_id, const char *assoc_obj_inst_variable, const char *txt_phrs);
void ROX_TMProtocol_Notify_Associative_Relationship_Deletion(int stmt_id, unsigned long dom_id, unsigned long rel_id, unsigned long from_obj_inst_id, const char *from_obj_inst_variable, unsigned long to_obj_inst_id, const char *to_obj_inst_variable, unsigned long assoc_obj_inst_id, const char *assoc_obj_inst_variable);
void ROX_TMProtocol_Notify_Bridge_Operation_Entry(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long brg_id, int num_params, unsigned long bparm_id[], const char *value[]);
void ROX_TMProtocol_Notify_Bridge_Operation_Exit(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long brg_id);
void ROX_TMProtocol_Notify_Bridge_Operation_Invocation(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long brg_id, int num_params, unsigned long bparm_id[], const char *oal_value[]);
void ROX_TMProtocol_Notify_Change_Expiry_of_Delayed_Event(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, int add, int delay);
void ROX_TMProtocol_Notify_Class_Operation_Entry(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id, int num_params, unsigned long tparm_id[], const char *value[]);
void ROX_TMProtocol_Notify_Class_Operation_Invocation(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id, int num_params, unsigned long tparm_id[], const char *oal_value[]);
void ROX_TMProtocol_Notify_Create_Class_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, int is_implicit, const char *local_variable, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[]);
void ROX_TMProtocol_Notify_Create_Creation_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long smevt_id, unsigned long evt_inst_id, int is_implicit, const char *local_variable, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[]);
void ROX_TMProtocol_Notify_Create_Delayed_Event(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, int delay, int recurring);
void ROX_TMProtocol_Notify_Create_External_Entity_Event(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long eeevt_id, unsigned long evt_inst_id, int num_params, unsigned long eeedi_id[], const char *value[], const char *oal_value[]);
void ROX_TMProtocol_Notify_Create_Object_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long obj_inst_id, unsigned long evt_inst_id, int is_implicit, const char *local_variable, const char *to_obj_inst_variable, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[]);
void ROX_TMProtocol_Notify_Create_Object_Instance_No_Variable(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, int num_params, unsigned long attr_id[], const char *value[]);
void ROX_TMProtocol_Notify_Create_Object_Instance(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, int is_implicit, const char *local_variable, int num_params, unsigned long attr_id[], const char *value[]);
void ROX_TMProtocol_Notify_Delayed_Event_Ready(unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, unsigned long recurring_evt_inst_id);
void ROX_TMProtocol_Notify_Delete_Delayed_Event(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id);
void ROX_TMProtocol_Notify_Delete_Event_Instance(unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id);
void ROX_TMProtocol_Notify_Delete_Object_Instance(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, const char *local_variable);
void ROX_TMProtocol_Notify_FBO_Entry(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long sm_id, unsigned long smstt_id, int num_params, unsigned long smedi_id[], const char *value[]);
void ROX_TMProtocol_Notify_FBO_Exit(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long sm_id, unsigned long smstt_id);
void ROX_TMProtocol_Notify_Function_Entry(int stmt_id, unsigned long dom_id, unsigned long sync_id, unsigned long funcinv_id, int num_params, unsigned long sparm_id[], const char *value[]);
void ROX_TMProtocol_Notify_Function_Exit(int stmt_id, unsigned long dom_id, unsigned long sync_id, char oal_invoked);
void ROX_TMProtocol_Notify_Function_Invocation(int stmt_id, unsigned long dom_id, unsigned long sync_id, int num_params, unsigned long sparm_id[], const char *oal_value[]);
void ROX_TMProtocol_Notify_Async_Function_Pending(unsigned long dom_id, unsigned long sync_id, unsigned long funcinv_id, int num_params, unsigned long sparm_id[], const char *value[]);
void ROX_TMProtocol_Notify_Async_Function_Executing(unsigned long dom_id, unsigned long sync_id, unsigned long funcinv_id);
void ROX_TMProtocol_Notify_Generate_Class_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[]);
void ROX_TMProtocol_Notify_Generate_Creation_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long smevt_id, unsigned long evt_inst_id, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[]);
void ROX_TMProtocol_Notify_Generate_Event_Attribute(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, unsigned long attr_id, const char *local_variable, unsigned long evt_inst_id);
void ROX_TMProtocol_Notify_Generate_Event_Variable(int stmt_id, const char *local_variable, unsigned long evt_inst_id);
void ROX_TMProtocol_Notify_Generate_External_Entity_Event(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long eeevt_id, unsigned long evt_inst_id, int num_params, unsigned long eeedi_id[], const char *value[], const char *oal_value[]);
void ROX_TMProtocol_Notify_Generate_Object_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long obj_inst_id, unsigned long evt_inst_id, const char *local_variable, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[]);
void ROX_TMProtocol_Notify_Instance_Operation_Entry(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id, unsigned long obj_inst_id, int num_params, unsigned long tparm_id[], const char *value[]);
void ROX_TMProtocol_Notify_Instance_Operation_Invocation(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id, unsigned long obj_inst_id, const char *local_variable, int num_params, unsigned long tparm_id[], const char *oal_value[]);
void ROX_TMProtocol_Notify_MDA_Invocation_Begin(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long attr_id);
void ROX_TMProtocol_Notify_MDA_Invocation_End(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long attr_id);
void ROX_TMProtocol_Notify_Modify_Attribute(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, unsigned long attr_id, const char *local_variable, const char *value, const char *oal_value);
void ROX_TMProtocol_Notify_Modify_Bridge_Operation_Parameter(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long brg_id, unsigned long bparm_id, const char *value, const char *oal_value);
void ROX_TMProtocol_Notify_Modify_Event_Data_Item(unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, unsigned long smedi_id, const char *value);
void ROX_TMProtocol_Notify_Modify_Function_Parameter(int stmt_id, unsigned long dom_id, unsigned long sync_id, unsigned long sparm_id, const char *value, const char *oal_value);
void ROX_TMProtocol_Notify_Modify_Local_Variable(int stmt_id, int is_implicit, const char *local_variable, const char *value, const char *oal_value);
void ROX_TMProtocol_Notify_Modify_Operation_Parameter(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id, unsigned long tparm_id, const char *value, const char *oal_value);
void ROX_TMProtocol_Notify_Modify_Referential_Attribute(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, unsigned long attr_id, const char *value, const char *local_variable);
void ROX_TMProtocol_Notify_Modify_Run_Mode(int run_mode);
void ROX_TMProtocol_Notify_Operation_Exit(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id);
void ROX_TMProtocol_Notify_Relationship_Creation(int stmt_id, unsigned long dom_id, unsigned long rel_id, unsigned long from_obj_inst_id, const char *from_obj_inst_variable, unsigned long to_obj_inst_id, const char *to_obj_inst_variable, const char *txt_phrs);
void ROX_TMProtocol_Notify_Relationship_Deletion(int stmt_id, unsigned long dom_id, unsigned long rel_id, unsigned long from_obj_inst_id, const char *from_obj_inst_variable, unsigned long to_obj_inst_id, const char *to_obj_inst_variable);
void ROX_TMProtocol_Notify_State_Entry(unsigned long dom_id, unsigned long sm_id, unsigned long smstt_id, unsigned long prev_smstt_id, unsigned long smevt_id, unsigned long evt_inst_id, unsigned long obj_inst_id, unsigned long orig_obj_inst_id);
void ROX_TMProtocol_Notify_State_Exit(unsigned long dom_id, unsigned long sm_id, unsigned long smstt_id, unsigned long obj_inst_id);
void ROX_TMProtocol_Notify_Activity_Pending(char * type);
void ROX_TMProtocol_Notify_Target_Shutdown(void);
void ROX_TMProtocol_Session_Done(void);
void ROX_TMProtocol_Session_Ready(const char *product, const char *version, const char *data, const char *protocol_version);
#endif


#ifdef	__cplusplus
}
#endif

#endif  /* TM_LAYER_H */

