/*---------------------------------------------------------------------------
 * File:  tm_layer.c
 *
 * Description:
 * The routines in this file provide a layer of indirection between
 * the fragment code and the UML Debugger protocol layer.
 *
 * This interface layer originally specified by Project Technology, Inc.
 *--------------------------------------------------------------------------*/
#ifdef ROX_TM_ENABLED
#include "sys_mechs.h"
#include "tm_layer.h"
#include "tm_socket.h"
#include "tm_pencode.h"
#include "tm_msghandlers.h"

static char * encbuf;
static int context;
static void ROX_Local_DebugMonitor_send( void );
static void ROX_Local_DebugMonitor_send( void )
{
  DebugMonitor_send( encbuf, Escher_strlen( encbuf ) );
  DebugMonitor_receive();
}

static void ROX_Local_Statement_Block( void );
static void ROX_Local_Statement_Block( void )
{
  if ( PROT_TM_run_mode() == 1 ) {
    DebugMonitor_block();
    context = 1;
  } else {
    context = 0;
  }
}

static void ROX_Local_State_Block( void );
static void ROX_Local_State_Block( void )
{
  if ( PROT_TM_run_mode() == 0 ) {
    DebugMonitor_block();
    context = 1;
  } else {
    context = 0;
  }
}

void ROX_TMProtocol_Add_Domain( unsigned long dom_id, const char *xml_url )
{
  PROT_TM_Add_Domain_encode(&encbuf, dom_id, xml_url);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Break_execution(int stmt_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Break_execution_encode(&encbuf, context, stmt_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Continue_execution(int stmt_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Continue_execution_encode(&encbuf, context, stmt_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Control_execution(int stmt_id, const char *command)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Control_execution_encode(&encbuf, context, stmt_id, command);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Creation_Transition(unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, int num_params, unsigned long attr_id[], const char *value[])
{
  PROT_TM_Log_Creation_Transition_encode(&encbuf, context, dom_id, obj_id, obj_inst_id, num_params, attr_id, value);
  ROX_Local_DebugMonitor_send();
}

bool ROX_TMProtocol_Log_Elif_execution(int stmt_id, const char *condition, bool eval)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Elif_execution_encode(&encbuf, context, stmt_id, condition, eval);
  ROX_Local_DebugMonitor_send();
  return eval;
}

void ROX_TMProtocol_Log_Else_execution(int stmt_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Else_execution_encode(&encbuf, context, stmt_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Endfor_execution(int stmt_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Endfor_execution_encode(&encbuf, context, stmt_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Endif_execution(int stmt_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Endif_execution_encode(&encbuf, context, stmt_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Endwhile_execution(int stmt_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Endwhile_execution_encode(&encbuf, context, stmt_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Event_Cant_Happen(unsigned long dom_id, unsigned long sm_id, unsigned long smstt_id, unsigned long obj_inst_id, unsigned long smevt_id, unsigned long evt_inst_id)
{
  PROT_TM_Log_Event_Cant_Happen_encode(&encbuf, context, dom_id, sm_id, smstt_id, obj_inst_id, smevt_id, evt_inst_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Event_Ignored(unsigned long dom_id, unsigned long sm_id, unsigned long smstt_id, unsigned long obj_inst_id, unsigned long smevt_id, unsigned long evt_inst_id)
{
  PROT_TM_Log_Event_Ignored_encode(&encbuf, context, dom_id, sm_id, smstt_id, obj_inst_id, smevt_id, evt_inst_id);
  ROX_Local_DebugMonitor_send();
}

bool ROX_TMProtocol_Log_For_execution(int stmt_id, const char *loop_variable_name, const char *set_variable_name, int is_implicit, bool eval)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_For_execution_encode(&encbuf, context, stmt_id, loop_variable_name, set_variable_name, is_implicit, eval);
  ROX_Local_DebugMonitor_send();
  return eval;
}

bool ROX_TMProtocol_Log_If_execution(int stmt_id, const char *condition, bool eval)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_If_execution_encode(&encbuf, context, stmt_id, condition, eval);
  ROX_Local_DebugMonitor_send();
  return eval;
}

void ROX_TMProtocol_Log_Occurrence( const char *severity, const char *message)
{
  PROT_TM_Log_Occurrence_encode(&encbuf, severity, message);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Return_execution(int stmt_id, const char *return_value, const char *oal_value)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Return_execution_encode(&encbuf, context, stmt_id, return_value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Select_from_execution(int stmt_id, const char *cardinality, const char *key_lett, int is_implicit, const char *local_variable)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Select_from_execution_encode(&encbuf, context, stmt_id, cardinality, key_lett, is_implicit, local_variable);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Select_from_where_execution(int stmt_id, const char *cardinality, const char *key_lett, const char *where_clause, int is_implicit, const char *local_variable)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Select_from_where_execution_encode(&encbuf, context, stmt_id, cardinality, key_lett, where_clause, is_implicit, local_variable);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Select_related_by_execution(int stmt_id, const char *cardinality, unsigned long obj_inst_id, const char *chain, int is_implicit, const char *local_variable)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Select_related_by_execution_encode(&encbuf, context, stmt_id, cardinality, obj_inst_id, chain, is_implicit, local_variable);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Log_Select_related_by_where_execution(int stmt_id, const char *cardinality, unsigned long obj_inst_id, const char *chain, const char *where_clause, int is_implicit, const char *local_variable)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_Select_related_by_where_execution_encode(&encbuf, context, stmt_id, cardinality, obj_inst_id, chain, where_clause, is_implicit, local_variable);
  ROX_Local_DebugMonitor_send();
}

bool ROX_TMProtocol_Log_While_execution(int stmt_id, const char *condition, bool eval)
{
  ROX_Local_Statement_Block();
  PROT_TM_Log_While_execution_encode(&encbuf, context, stmt_id, condition, eval);
  ROX_Local_DebugMonitor_send();
  return eval;
}

void ROX_TMProtocol_Notify_Associative_Relationship_Creation(int stmt_id, unsigned long dom_id, unsigned long rel_id, unsigned long from_obj_inst_id, const char *from_obj_inst_variable, unsigned long to_obj_inst_id, const char *to_obj_inst_variable, unsigned long assoc_obj_inst_id, const char *assoc_obj_inst_variable, const char *txt_phrs)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Associative_Relationship_Creation_encode(&encbuf, context, stmt_id, dom_id, rel_id, from_obj_inst_id, from_obj_inst_variable, to_obj_inst_id, to_obj_inst_variable, assoc_obj_inst_id, assoc_obj_inst_variable, txt_phrs);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Associative_Relationship_Deletion(int stmt_id, unsigned long dom_id, unsigned long rel_id, unsigned long from_obj_inst_id, const char *from_obj_inst_variable, unsigned long to_obj_inst_id, const char *to_obj_inst_variable, unsigned long assoc_obj_inst_id, const char *assoc_obj_inst_variable)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Associative_Relationship_Deletion_encode(&encbuf, context, stmt_id, dom_id, rel_id, from_obj_inst_id, from_obj_inst_variable, to_obj_inst_id, to_obj_inst_variable, assoc_obj_inst_id, assoc_obj_inst_variable);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Bridge_Operation_Entry(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long brg_id, int num_params, unsigned long bparm_id[], const char *value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Bridge_Operation_Entry_encode(&encbuf, context, stmt_id, dom_id, ee_id, brg_id, num_params, bparm_id, value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Bridge_Operation_Exit(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long brg_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Bridge_Operation_Exit_encode(&encbuf, context, stmt_id, dom_id, ee_id, brg_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Bridge_Operation_Invocation(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long brg_id, int num_params, unsigned long bparm_id[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Bridge_Operation_Invocation_encode(&encbuf, context, stmt_id, dom_id, ee_id, brg_id, num_params, bparm_id, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Change_Expiry_of_Delayed_Event(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, int add, int delay)
{
  PROT_TM_Notify_Change_Expiry_of_Delayed_Event_encode(&encbuf, context, stmt_id, dom_id, sm_id, smevt_id, evt_inst_id, add, delay);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Class_Operation_Entry(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id, int num_params, unsigned long tparm_id[], const char *value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Class_Operation_Entry_encode(&encbuf, context, stmt_id, dom_id, obj_id, tfr_id, num_params, tparm_id, value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Class_Operation_Invocation(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id, int num_params, unsigned long tparm_id[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Class_Operation_Invocation_encode(&encbuf, context, stmt_id, dom_id, obj_id, tfr_id, num_params, tparm_id, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Create_Class_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, int is_implicit, const char *local_variable, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Create_Class_Event_Instance_encode(&encbuf, context, stmt_id, dom_id, sm_id, smevt_id, evt_inst_id, is_implicit, local_variable, num_params, smedi_id, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Create_Creation_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long smevt_id, unsigned long evt_inst_id, int is_implicit, const char *local_variable, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Create_Creation_Event_Instance_encode(&encbuf, context, stmt_id, dom_id, obj_id, smevt_id, evt_inst_id, is_implicit, local_variable, num_params, smedi_id, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Create_Delayed_Event(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, int delay, int recurring)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Create_Delayed_Event_encode(&encbuf, context, stmt_id, dom_id, sm_id, smevt_id, evt_inst_id, delay, recurring);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Create_External_Entity_Event(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long eeevt_id, unsigned long evt_inst_id, int num_params, unsigned long eeedi_id[], const char *value[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Create_External_Entity_Event_encode(&encbuf, context, stmt_id, dom_id, ee_id, eeevt_id, evt_inst_id, num_params, eeedi_id, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Create_Object_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long obj_inst_id, unsigned long evt_inst_id, int is_implicit, const char *local_variable, const char *to_obj_inst_variable, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Create_Object_Event_Instance_encode(&encbuf, context, stmt_id, dom_id, sm_id, smevt_id, obj_inst_id, evt_inst_id, is_implicit, local_variable, to_obj_inst_variable, num_params, smedi_id, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Create_Object_Instance_No_Variable(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, int num_params, unsigned long attr_id[], const char *value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Create_Object_Instance_No_Variable_encode(&encbuf, context, stmt_id, dom_id, obj_id, obj_inst_id, num_params, attr_id, value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Create_Object_Instance(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, int is_implicit, const char *local_variable, int num_params, unsigned long attr_id[], const char *value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Create_Object_Instance_encode(&encbuf, context, stmt_id, dom_id, obj_id, obj_inst_id, is_implicit, local_variable, num_params, attr_id, value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Delayed_Event_Ready( unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, unsigned long recurring_evt_inst_id)
{
  PROT_TM_Notify_Delayed_Event_Ready_encode(&encbuf, dom_id, sm_id, smevt_id, evt_inst_id, recurring_evt_inst_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Delete_Delayed_Event(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id)
{
  PROT_TM_Notify_Delete_Delayed_Event_encode(&encbuf, context, stmt_id, dom_id, sm_id, smevt_id, evt_inst_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Delete_Event_Instance(unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id)
{
  PROT_TM_Notify_Delete_Event_Instance_encode(&encbuf, context, dom_id, sm_id, smevt_id, evt_inst_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Delete_Object_Instance(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, const char *local_variable)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Delete_Object_Instance_encode(&encbuf, context, stmt_id, dom_id, obj_id, obj_inst_id, local_variable);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_FBO_Entry(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long sm_id, unsigned long smstt_id, int num_params, unsigned long smedi_id[], const char *value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_FBO_Entry_encode(&encbuf, context, stmt_id, dom_id, obj_id, sm_id, smstt_id, num_params, smedi_id, value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_FBO_Exit(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long sm_id, unsigned long smstt_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_FBO_Exit_encode(&encbuf, context, stmt_id, dom_id, obj_id, sm_id, smstt_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Function_Entry(int stmt_id, unsigned long dom_id, unsigned long sync_id, unsigned long funcinv_id, int num_params, unsigned long sparm_id[], const char *value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Function_Entry_encode(&encbuf, context, stmt_id, dom_id, sync_id, funcinv_id, num_params, sparm_id, value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Function_Exit(int stmt_id, unsigned long dom_id, unsigned long sync_id, char oal_invoked)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Function_Exit_encode(&encbuf, context, stmt_id, dom_id, sync_id, oal_invoked);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Function_Invocation(int stmt_id, unsigned long dom_id, unsigned long sync_id, int num_params, unsigned long sparm_id[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Function_Invocation_encode(&encbuf, context, stmt_id, dom_id, sync_id, num_params, sparm_id, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Async_Function_Pending(unsigned long dom_id, unsigned long sync_id, unsigned long funcinv_id, int num_params, unsigned long sparm_id[], const char *value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Async_Function_Pending_encode(&encbuf, dom_id, sync_id, funcinv_id, num_params, sparm_id, value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Async_Function_Executing(unsigned long dom_id, unsigned long sync_id, unsigned long funcinv_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Async_Function_Executing_encode(&encbuf, context, dom_id, sync_id, funcinv_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Generate_Class_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Generate_Class_Event_Instance_encode(&encbuf, context, stmt_id, dom_id, sm_id, smevt_id, evt_inst_id, num_params, smedi_id, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Generate_Creation_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long smevt_id, unsigned long evt_inst_id, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Generate_Creation_Event_Instance_encode(&encbuf, context, stmt_id, dom_id, obj_id, smevt_id, evt_inst_id, num_params, smedi_id, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Generate_Event_Attribute(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, unsigned long attr_id, const char *local_variable, unsigned long evt_inst_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Generate_Event_Attribute_encode(&encbuf, context, stmt_id, dom_id, obj_id, obj_inst_id, attr_id, local_variable, evt_inst_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Generate_Event_Variable(int stmt_id, const char *local_variable, unsigned long evt_inst_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Generate_Event_Variable_encode(&encbuf, context, stmt_id, local_variable, evt_inst_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Generate_External_Entity_Event(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long eeevt_id, unsigned long evt_inst_id, int num_params, unsigned long eeedi_id[], const char *value[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Generate_External_Entity_Event_encode(&encbuf, context, stmt_id, dom_id, ee_id, eeevt_id, evt_inst_id, num_params, eeedi_id, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Generate_Object_Event_Instance(int stmt_id, unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long obj_inst_id, unsigned long evt_inst_id, const char *local_variable, int num_params, unsigned long smedi_id[], const char *value[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Generate_Object_Event_Instance_encode(&encbuf, context, stmt_id, dom_id, sm_id, smevt_id, obj_inst_id, evt_inst_id, local_variable, num_params, smedi_id, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Instance_Operation_Entry(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id, unsigned long obj_inst_id, int num_params, unsigned long tparm_id[], const char *value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Instance_Operation_Entry_encode(&encbuf, context, stmt_id, dom_id, obj_id, tfr_id, obj_inst_id, num_params, tparm_id, value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Instance_Operation_Invocation(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id, unsigned long obj_inst_id, const char *local_variable, int num_params, unsigned long tparm_id[], const char *oal_value[])
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Instance_Operation_Invocation_encode(&encbuf, context, stmt_id, dom_id, obj_id, tfr_id, obj_inst_id, local_variable, num_params, tparm_id, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_MDA_Invocation_Begin(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long attr_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_MDA_Invocation_Begin_encode(&encbuf, context, stmt_id, dom_id, obj_id, attr_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_MDA_Invocation_End(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long attr_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_MDA_Invocation_End_encode(&encbuf, context, stmt_id, dom_id, obj_id, attr_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Modify_Attribute(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, unsigned long attr_id, const char *local_variable, const char *value, const char *oal_value)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Modify_Attribute_encode(&encbuf, context, stmt_id, dom_id, obj_id, obj_inst_id, attr_id, local_variable, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Modify_Bridge_Operation_Parameter(int stmt_id, unsigned long dom_id, unsigned long ee_id, unsigned long brg_id, unsigned long bparm_id, const char *value, const char *oal_value)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Modify_Bridge_Operation_Parameter_encode(&encbuf, context, stmt_id, dom_id, ee_id, brg_id, bparm_id, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Modify_Event_Data_Item(unsigned long dom_id, unsigned long sm_id, unsigned long smevt_id, unsigned long evt_inst_id, unsigned long smedi_id, const char *value)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Modify_Event_Data_Item_encode(&encbuf, context, dom_id, sm_id, smevt_id, evt_inst_id, smedi_id, value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Modify_Function_Parameter(int stmt_id, unsigned long dom_id, unsigned long sync_id, unsigned long sparm_id, const char *value, const char *oal_value)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Modify_Function_Parameter_encode(&encbuf, context, stmt_id, dom_id, sync_id, sparm_id, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Modify_Local_Variable(int stmt_id, int is_implicit, const char *local_variable, const char *value, const char *oal_value)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Modify_Local_Variable_encode(&encbuf, context, stmt_id, is_implicit, local_variable, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Modify_Operation_Parameter(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id, unsigned long tparm_id, const char *value, const char *oal_value)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Modify_Operation_Parameter_encode(&encbuf, context, stmt_id, dom_id, obj_id, tfr_id, tparm_id, value, oal_value);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Modify_Referential_Attribute(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long obj_inst_id, unsigned long attr_id, const char *value, const char *local_variable)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Modify_Referential_Attribute_encode(&encbuf, context, stmt_id, dom_id, obj_id, obj_inst_id, attr_id, value, local_variable);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Modify_Run_Mode( int run_mode)
{
  PROT_TM_Notify_Modify_Run_Mode_encode(&encbuf, run_mode);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Operation_Exit(int stmt_id, unsigned long dom_id, unsigned long obj_id, unsigned long tfr_id)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Operation_Exit_encode(&encbuf, context, stmt_id, dom_id, obj_id, tfr_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Relationship_Creation(int stmt_id, unsigned long dom_id, unsigned long rel_id, unsigned long from_obj_inst_id, const char *from_obj_inst_variable, unsigned long to_obj_inst_id, const char *to_obj_inst_variable, const char *txt_phrs)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Relationship_Creation_encode(&encbuf, context, stmt_id, dom_id, rel_id, from_obj_inst_id, from_obj_inst_variable, to_obj_inst_id, to_obj_inst_variable, txt_phrs);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Relationship_Deletion(int stmt_id, unsigned long dom_id, unsigned long rel_id, unsigned long from_obj_inst_id, const char *from_obj_inst_variable, unsigned long to_obj_inst_id, const char *to_obj_inst_variable)
{
  ROX_Local_Statement_Block();
  PROT_TM_Notify_Relationship_Deletion_encode(&encbuf, context, stmt_id, dom_id, rel_id, from_obj_inst_id, from_obj_inst_variable, to_obj_inst_id, to_obj_inst_variable);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_State_Entry(unsigned long dom_id, unsigned long sm_id, unsigned long smstt_id, unsigned long prev_smstt_id, unsigned long smevt_id, unsigned long evt_inst_id, unsigned long obj_inst_id, unsigned long orig_obj_inst_id)
{
  PROT_TM_Notify_State_Entry_encode(&encbuf, context, dom_id, sm_id, smstt_id, prev_smstt_id, smevt_id, evt_inst_id, obj_inst_id, orig_obj_inst_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_State_Exit(unsigned long dom_id, unsigned long sm_id, unsigned long smstt_id, unsigned long obj_inst_id)
{
  PROT_TM_Notify_State_Exit_encode(&encbuf, context, dom_id, sm_id, smstt_id, obj_inst_id);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Activity_Pending( char * type )
{
  ROX_Local_State_Block();
  PROT_TM_Notify_Activity_Pending_encode(&encbuf, context, type);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Notify_Target_Shutdown( void )
{
  PROT_TM_Notify_Target_Shutdown_encode(&encbuf);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Session_Done( void )
{
  PROT_TM_Session_Done_encode(&encbuf);
  ROX_Local_DebugMonitor_send();
}

void ROX_TMProtocol_Session_Ready( const char *product, const char *version, const char *data, const char *protocol_version)
{
  PROT_TM_Session_Ready_encode(&encbuf, product, version, data, protocol_version);
  ROX_Local_DebugMonitor_send();
}
#endif


