/*---------------------------------------------------------------------------
 * File:  tm_socket.c
 *
 * Description:
 * This file contains the link level layer of communiction between
 * the UML Debugger and the target monitor.  The link provided here
 * is a socket based link.
 *
 *
 * Notice:
 *   (C) Copyright 1999-2003 ROX Software, Inc.
 *   All rights reserved.
 *
 * Model Compiler:  MC3020 V3.3.0
 * Serial Number:   302033003113005
 *
 * Warnings:
 *   !!! THIS IS AN AUTO-GENERATED FILE. PLEASE DO NOT EDIT. !!!
 * Based on ptc_TMdebug_monitor.cpp from Project Technology, Inc.
 *--------------------------------------------------------------------------*/
#ifdef ROX_TM_ENABLED

#ifdef ROX_TM_MVC
#include <windows.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#endif
#include "sys_debug.h"
#include "tm_socket.h"
#include "TIM_bridge.h"

/*-------------------------------------------------------------------*/
/* Local static member data is here.                                 */
/*-------------------------------------------------------------------*/
static struct hostent *hp;              /* server host structure     */
static struct sockaddr_in server;       /* server socket structure   */
static char hostname[ 64 ];             /* name of host running UMLD */
static unsigned int port;               /* TCP port of UMLD server   */
static ROX_TMcallback_t callback;       /* handler for message       */
static char buffer[ 512 ];              /* buffer for messages       */
static fd_set socklist;                 /* socket descriptor set     */
#ifdef ROX_TM_MVC
static WSADATA windows_socket_data;     /* MS Windows sockets v2     */
static SOCKET s;                        /* Windows socket handle     */
#else
static int s;                           /* socket (file) handle      */
#endif


/*-------------------------------------------------------------------*/
/* Call this routine to connect to the UML debugger and begin        */
/* processing messages from the host.                                */
/*-------------------------------------------------------------------*/
void DebugMonitor_process( void )
{
  /* Populate the host structure with information about the host     */
  /* running the UML debugger.  */
  if ( ( hp = gethostbyname( hostname ) ) == 0 ) {
    /* Bad news.  Cannot build host structure.  */
    return;
  }
  /* Populate the socket structure first with host information.      */
  Escher_memset( &server, 0, sizeof(server) );
  Escher_memmove( &( server.sin_addr ), hp->h_addr_list[0], hp->h_length );
  server.sin_family = hp->h_addrtype;
  server.sin_port = htons(port);

  /* Create the socket.  */
  s = socket( AF_INET, SOCK_STREAM, 0 );
  if ( s < 0 ) {
    /* Bad news.  Cannot build socket.  */
    return;
  }

  DebugMonitor_block();   /* Start out blocking on reads.  */

  /* Connect to the server.  Wait forever until server accepts.  */
  while( connect( s, (const struct sockaddr *) &server, sizeof(server) ) != 0 )
    ;
}


/*-------------------------------------------------------------------*/
/* Attempt to read a message.  One may be there or not.  If so,      */
/* we read it and process it.  Otherwise, plan to come back later.   */
/* This routine needs to be called periodically (perhaps from the    */
/* background loop.  The callback handler will be invoked when a     */
/* message arrives.                                                  */
/*-------------------------------------------------------------------*/
int DebugMonitor_receive( void )
{
  int retval = 0;
  struct timeval timeout;
  /* Do not bother receiving a message if no handler exists for it.  */
  if ( callback != 0 ) {
    if ( ROX_TM_blocking_bit == 1 ) {
      TIM_pause();      /* Freeze timers while we talk to debugger.  */
    }
    /*---------------------------------------------------------------*/
    /* Use select to check for receive data.  Set the blocking       */
    /* timeout based on whether we want to wait on the host          */
    /* (debugger) or not.                                            */
    /*---------------------------------------------------------------*/
    FD_ZERO( &socklist );
    FD_SET( s, &socklist );
    timeout.tv_usec = 0;
    timeout.tv_sec = ( ROX_TM_blocking_bit == 1 ) ? 30000 : 0;
    retval = select ( s + 1, &socklist, 0, 0, &timeout );
    if ( retval > 0 ) {
      retval = recv( s, buffer, sizeof(buffer), 0 );
      /* Process received message.   */
      ( *callback )( buffer );
    }
    if ( ROX_TM_blocking_bit == 1 ) {
      TIM_resume();     /* Restart ticker.                           */
    }
  }
  return retval;
}


/*-------------------------------------------------------------------*/
/* Attempt to send a message to the host through the socket.         */
/* Only length bytes will be transmitted even if more exist.         */
/* Note that this routine walks on the input buffer.                 */
/*-------------------------------------------------------------------*/
void DebugMonitor_send(
  char * message,
  int length
)
{
  message[ length ] = '\n';
  ++length;
  message[ length ] = 0;
  if ( send( s, message, length, 0 ) < 0 ) {
    /* Bad news.  Cannot send message.  */
    return;
  }
}

/*-------------------------------------------------------------------*/
/* Call this routine to turn on blocking reads.                      */
/*-------------------------------------------------------------------*/
void DebugMonitor_block( void )
{
  /* Set the socket flag to provide blocking reads and writes.       */
  ROX_TM_blocking_bit = 1;
}

/*-------------------------------------------------------------------*/
/* Call this routine to turn off blocking reads.                     */
/*-------------------------------------------------------------------*/
void DebugMonitor_noblock( void )
{
  /* Set the socket flag to provide non-blocking reads and writes.   */
  ROX_TM_blocking_bit = 0;
}

/*-------------------------------------------------------------------*/
/* Initialize the debug monitor with the server hostname.            */
/*-------------------------------------------------------------------*/
void DebugMonitor_init(
  const char * h,
  const unsigned int p,
  ROX_TMcallback_t cb
)
{
  Escher_strcpy( hostname, h );
  port = p;
  callback = cb;
#ifdef ROX_TM_MVC
  WSAStartup( MAKEWORD(2, 2), &windows_socket_data );
#endif
}


/*-------------------------------------------------------------------*/
/* Wipe out any useful information in case someone tries to use      */
/* it incorrectly.                                                   */
/*-------------------------------------------------------------------*/
void DebugMonitor_shutdown( void )
{
  shutdown( s, 2 );                /* Shut down the socket.          */
  buffer[ 0 ] = 0;
  hostname[ 0 ] = 0;
  port = 0;
  callback = 0;
  s = 0;
}
#endif


