/*---------------------------------------------------------------------------
 * File:  tm_xmlparser.c
 *
 * Description:
 * This is the debug message parser.  The grammar for the input that
 * is parsable is provided below.
 *
 *
 * Notice:
 *   (C) Copyright 1999-2003 ROX Software, Inc.
 *   All rights reserved.
 *
 * Model Compiler:  MC3020 V3.3.0
 * Serial Number:   302033003113005
 *
 * Warnings:
 *   !!! THIS IS AN AUTO-GENERATED FILE. PLEASE DO NOT EDIT. !!!
 *--------------------------------------------------------------------------*/
#ifdef ROX_TM_ENABLED
/*

   XML Parser Grammar


   debug_entity   : "<debug" ( attribute )* ( "/>" | more )

   more           : ">" ( subentity )* "</debug>"

   attribute      : attribute_name "=" stringvalue

   subentity      : value | param

   value          : "<value" ( attribute )* ">" data "</value>"

   param          : "<param>" ( attribute )* data "</param>"

   attribute_name : [a-zA-z][a-zA-Z0-9_-]*

   data           : [^<]*

   stringvalue    : '"' [^"]* '"'

*/

#include "sys_mechs.h"
#include "tm_xmlparser.h"

static bool debug_entity( void );
static bool more( void );
static bool attribute( void );
static bool subentity( void );
static bool value( void );
static bool param( void );
static bool attribute_name( void );
static bool data( void );
static bool stringvalue( void );
static void whitespace( void );
static bool parsestring( const c_t * );
static void dmsg_init( void );

/*--------------------------------------------------------------------*/
/* Declare the shape of a name/value pair (nvp) type.                 */
/*--------------------------------------------------------------------*/
typedef struct {
  const c_t * name;
  const c_t * value;
  const c_t * data;
} nvp_t;

/*--------------------------------------------------------------------*/
/* Declare the shape of the storare for the XML data.                 */
/*--------------------------------------------------------------------*/
#define DEBUG_IVP_MAX 16
typedef struct {
  s1_t nvpindex;             /* DEBUG_IVP_MAX must be less than 128.  */
  nvp_t nvp[ DEBUG_IVP_MAX ];
} debug_message_t;

#define DEBUG_MSG_MAX 2
static debug_message_t dmsg[ DEBUG_MSG_MAX ];
static c_t * in;
static const c_t * cursor;
static s1_t dmsgindex;       /* DEBUG_MSG_MAX must be less than 128.  */

/*
#define DEVELOPER_DEBUG( s, c ) printf( s, c );
*/
#define DEVELOPER_DEBUG( s, c )

/*--------------------------------------------------------------------*/
/* Parse a message returning the command and a pointer to a data      */
/* buffer.                                                            */
/*--------------------------------------------------------------------*/
c_t PROT_parse_message(
  const c_t * message,
  c_t * name,
  void ** d
)
{
  u1_t j;
  dmsg_init();               /* Initialize the xml storage area.      */
  cursor = message;
  in = (c_t *) *d;

  /* Parse all messages in the buffer.  */
  while ( debug_entity() && ( dmsgindex < DEBUG_MSG_MAX ) ) {
    /* Find the command among the attribute name value pairs.         */
    for ( j = 0; j <= dmsg[ dmsgindex ].nvpindex; j++ ) {
      if ( Escher_strcmp( dmsg[ dmsgindex ].nvp[ j ].name, "cmd" ) == 0 ) {
        Escher_strcpy( name, dmsg[ dmsgindex ].nvp[ j ].value );
        break;
      }
    }
  }
  /* We somehow need to return the xml data to the application.  */
  return dmsgindex + 1;
}

/*--------------------------------------------------------------------*/
/* Clear out the message structure.                                   */
/* Only here do dmsgindex and nvpindex get reset.                     */
/*--------------------------------------------------------------------*/
static void dmsg_init( void )
{
  u1_t i, j;
  dmsgindex = -1;
  for ( i = 0; i < DEBUG_MSG_MAX; i++ ) {
    dmsg[ i ].nvpindex = -1;
    for ( j = 0; j < DEBUG_IVP_MAX; j++ ) {
      dmsg[ i ].nvp[ j ].name = 0;
      dmsg[ i ].nvp[ j ].value = 0;
      dmsg[ i ].nvp[ j ].data = 0;
    }
  }
}

/*--------------------------------------------------------------------*/
/* Return the value of the named attribute if it exists in the        */
/* message structure.  Return null ( 0 ) if it does not exist.        */
/*--------------------------------------------------------------------*/
const c_t * PROT_get_attribute_value(
  const c_t * aname
)
{
  u1_t i, j;
  for ( i = 0; i <= dmsgindex; i++ ) {
    /* Find the attribute among the attribute name value pairs.       */
    for ( j = 0; j <= dmsg[ i ].nvpindex; j++ ) {
      if ( Escher_strcmp( dmsg[ i ].nvp[ j ].name, aname ) == 0 ) {
        return dmsg[ i ].nvp[ j ].value;
      }
    }
  }
  return 0;
}


/*--------------------------------------------------------------------*/
/* Return the data for the supplied attribute value if it exists in   */
/* the message structure.  Return null ( 0 ) if it does not exist.    */
/*--------------------------------------------------------------------*/
const c_t * PROT_get_attribute_data(
  const c_t * aname,
  const c_t * avalue
)
{
  u1_t i, j;
  for ( i = 0; i <= dmsgindex; i++ ) {
    /* Find the data among the attribute name value pairs.         */
    for ( j = 0; j <= dmsg[ i ].nvpindex; j++ ) {
      if ( ( Escher_strcmp( dmsg[ i ].nvp[ j ].name, aname ) == 0 ) &&
        ( Escher_strcmp( dmsg[ i ].nvp[ j ].value, avalue ) == 0 ) ) {
        return dmsg[ i ].nvp[ j ].data;
      }
    }
  }
  return 0;
}


/*--------------------------------------------------------------------*/
/* Parse the <debug> XML entity.  Stow the interesting components     */
/* into a data structure for access by the application.               */
/*--------------------------------------------------------------------*/
static bool debug_entity( void )
{
  DEVELOPER_DEBUG( "debug_entity %s\n", cursor );
  if ( ! parsestring( "<debug" ) ) return false;
  dmsgindex++;           /* Increment dmsgindex here and only here.   */
  while ( attribute() )
    ;
  if ( ( parsestring( "/>" ) ) || ( more() ) ) return true;
  return false;
}

static bool more( void )
{
  DEVELOPER_DEBUG( "more() %s\n", cursor );
  if ( ! parsestring( ">" ) ) return false;
  while ( subentity() )
    ;
  if ( ! parsestring( "</debug>" ) ) return false;
  return true;
}

static bool attribute( void )
{
  DEVELOPER_DEBUG( "attribute() %s\n", cursor );
  if ( ! attribute_name() ) return false;
  if ( ! parsestring( "=" ) ) return false;
  if ( ! stringvalue() ) return false;
  return true;
}

static bool subentity( void )
{
  DEVELOPER_DEBUG( "subentity() %s\n", cursor );
  while ( value() || param() )
    ;
  return false;
}

static bool value( void )
{
  DEVELOPER_DEBUG( "value() %s\n", cursor );
  if ( ! parsestring( "<value" ) ) return false;
  while ( attribute() )
    ;
  if ( ! parsestring( ">" ) ) return false;
  if ( ! data() ) return false;
  if ( ! parsestring( "</value>" ) ) return false;
  return true;
}
  
static bool param( void )
{
  DEVELOPER_DEBUG( "param() %s\n", cursor );
  if ( ! parsestring( "<param>" ) ) return false;
  while ( attribute() )
    ;
  if ( ! data() ) return false;
  if ( ! parsestring( "</param>" ) ) return false;
  return true;
}

static bool attribute_name( void )
{
  DEVELOPER_DEBUG( "attribute_name() %s\n", cursor );
  whitespace();
  if ( ! ( ( ( 'a' <= *cursor ) && ( *cursor <= 'z' ) ) ||
           ( ( 'A' <= *cursor ) && ( *cursor <= 'Z' ) ) ) ) {
    return false;
  }
  dmsg[ dmsgindex ].nvpindex++;  /* Increment to new nvpindex here.  */
  /* Stow away the attribute name.  */
  dmsg[ dmsgindex ].nvp[ dmsg[ dmsgindex ].nvpindex ].name = in;
  *in++ = *cursor++;
  while ( ( ( 'a' <= *cursor ) && ( *cursor <= 'z' ) ) ||
          ( ( 'A' <= *cursor ) && ( *cursor <= 'Z' ) ) ||
          ( ( '0' <= *cursor ) && ( *cursor <= '9' ) ) ||
          ( *cursor == '_' ) ||
          ( *cursor == '-' ) ) {
    *in++ = *cursor++;
  }
  *in++ = 0;
  return true;
}

static bool data( void )
{
  u1_t guard_counter = 0;
  DEVELOPER_DEBUG( "data() %s\n", cursor );
  /* Stow away the attribute data.  */
  dmsg[ dmsgindex ].nvp[ dmsg[ dmsgindex ].nvpindex ].data = in;
  while ( ( *cursor != '<' ) && ( guard_counter++ < 250 ) ) {
    *in++ = *cursor++;
  }
  *in++ = 0;
  return true;
}

static bool stringvalue( void )
{
  u1_t guard_counter = 0;
  DEVELOPER_DEBUG( "stringvalue() %s\n", cursor );
  whitespace();
  if ( *cursor++ != '"' ) return false;
  /* Stow away the attribute value.  */
  dmsg[ dmsgindex ].nvp[ dmsg[ dmsgindex ].nvpindex ].value = in;
  while ( ( *cursor != '"' ) && ( guard_counter++ < 250 ) ) {
    *in++ = *cursor++;
  }
  cursor++;   /* Eat the trailing quotation mark.  */
  *in++ = 0;
  return true;
}

static bool parsestring(
  const c_t * s1
)
{
  const c_t * s2;
  whitespace();
  s2 = cursor;
  while ( *s1++ == *s2++ ) {
    if ( *s1 == 0 ) {
      cursor = s2;
      return true;
    }
  }
  return false;
}

static void whitespace( void )
{
  u1_t guard_counter = 0;
  while ( ( *cursor == ' ' ) && ( guard_counter++ < 250 ) ) {
    cursor++;
  }
}
#endif


